@@ -10,7 +10,7 @@ use std::{
 
 use crate::{err::ConfigError, ConfigContext, FromConfig};
 
-/// Config value, [ConfigSource](./trait.ConfigSource.html) use this value to store config properties.
+/// Config value, [ConfigSource](source/trait.ConfigSource.html) use this value to store config properties.
 ///
 /// # Placeholder expression
 ///
@@ -42,6 +42,59 @@ pub enum ConfigValue<'a> {
     Float(f64),
     /// Bool value.
     Bool(bool),
+    #[cfg(feature = "rand")]
+    /// Random value.
+    Rand(RandValue),
+}
+
+#[doc(hidden)]
+#[cfg(feature = "rand")]
+#[derive(Debug, Clone, Copy)]
+pub enum RandValue {
+    U8,
+    U16,
+    U32,
+    U64,
+    U128,
+    Usize,
+    I8,
+    I16,
+    I32,
+    I64,
+    I128,
+    Isize,
+}
+
+impl ConfigValue<'_> {
+    pub(crate) fn clone_static(&self) -> ConfigValue<'static> {
+        match self {
+            ConfigValue::StrRef(v) => ConfigValue::Str(v.to_string()),
+            ConfigValue::Str(v) => ConfigValue::Str(v.to_string()),
+            ConfigValue::Int(v) => ConfigValue::Int(*v),
+            ConfigValue::Float(v) => ConfigValue::Float(*v),
+            ConfigValue::Bool(v) => ConfigValue::Bool(*v),
+            #[cfg(feature = "rand")]
+            ConfigValue::Rand(v) => ConfigValue::Rand(*v),
+        }
+    }
+
+    #[cfg(feature = "rand")]
+    pub(crate) fn normalize(v: RandValue) -> Self {
+        match v {
+            RandValue::U8 => ConfigValue::Int(rand::random::<u8>() as i64),
+            RandValue::U16 => ConfigValue::Int(rand::random::<u16>() as i64),
+            RandValue::U32 => ConfigValue::Int(rand::random::<u32>() as i64),
+            RandValue::U64 => ConfigValue::Str(rand::random::<u64>().to_string()),
+            RandValue::U128 => ConfigValue::Str(rand::random::<u128>().to_string()),
+            RandValue::Usize => ConfigValue::Str(rand::random::<usize>().to_string()),
+            RandValue::I8 => ConfigValue::Int(rand::random::<i8>() as i64),
+            RandValue::I16 => ConfigValue::Int(rand::random::<i16>() as i64),
+            RandValue::I32 => ConfigValue::Int(rand::random::<i32>() as i64),
+            RandValue::I64 => ConfigValue::Int(rand::random::<i64>()),
+            RandValue::I128 => ConfigValue::Str(rand::random::<i128>().to_string()),
+            RandValue::Isize => ConfigValue::Str(rand::random::<isize>().to_string()),
+        }
+    }
 }
 
 impl<'a> Into<ConfigValue<'a>> for String {
@@ -56,22 +109,48 @@ impl<'a> Into<ConfigValue<'a>> for &'a str {
     }
 }
 
-impl<'a> Into<ConfigValue<'a>> for i64 {
-    fn into(self) -> ConfigValue<'a> {
-        ConfigValue::Int(self)
-    }
+macro_rules! into_config_value_le {
+    ($f:ident=$t:ident: $($x:ident),*) => {$(
+        impl<'a> Into<ConfigValue<'a>> for $x {
+            #[allow(trivial_numeric_casts)]
+            fn into(self) -> ConfigValue<'a> {
+                ConfigValue::$f(self as $t)
+            }
+        })*
+    };
 }
-impl<'a> Into<ConfigValue<'a>> for f64 {
-    fn into(self) -> ConfigValue<'a> {
-        ConfigValue::Float(self)
-    }
+
+into_config_value_le!(Int = i64: u8, u16, u32, i8, i16, i32, i64);
+into_config_value_le!(Float = f64: f32, f64);
+
+macro_rules! into_config_value {
+    ($($x:ident),*) => {$(
+        impl<'a> Into<ConfigValue<'a>> for $x {
+            fn into(self) -> ConfigValue<'a> {
+                if self <= i64::MAX as $x {
+                    return ConfigValue::Int(self as i64);
+                }
+                ConfigValue::Str(self.to_string())
+            }
+        })*
+    };
 }
+
+into_config_value!(u64, u128, usize, i128, isize);
+
 impl<'a> Into<ConfigValue<'a>> for bool {
     fn into(self) -> ConfigValue<'a> {
         ConfigValue::Bool(self)
     }
 }
 
+#[cfg(feature = "rand")]
+impl<'a> Into<ConfigValue<'a>> for RandValue {
+    fn into(self) -> ConfigValue<'a> {
+        ConfigValue::Rand(self)
+    }
+}
+
 impl<V: FromConfig> FromConfig for Result<V, ConfigError> {
     fn from_config(
         context: &mut ConfigContext<'_>,
@@ -165,6 +244,8 @@ impl FromValue for String {
             ConfigValue::Int(s) => s.to_string(),
             ConfigValue::Float(s) => check_f64(context, s)?.to_string(),
             ConfigValue::Bool(s) => s.to_string(),
+            #[cfg(feature = "rand")]
+            _ => return Err(context.parse_error("ConfigValueError")),
         };
         Ok(v)
     }
@@ -398,7 +479,7 @@ mod test {
             &mut self,
             val: impl Into<ConfigValue<'a>>,
         ) -> Result<T, ConfigError> {
-            T::from_value(&mut self.0.new_context(&mut self.1), val.into())
+            T::from_value(&mut self.0.source.new_context(&mut self.1), val.into())
         }
     }
 
@@ -466,10 +547,10 @@ mod test {
         should_err!(context: "f" as bool);
         should_err!(context: "y" as bool);
         should_err!(context: "t" as bool);
-        should_err!(context: 0 as bool);
-        should_err!(context: 1 as bool);
-        should_err!(context: 0.0 as bool);
-        should_err!(context: 1.0 as bool);
+        should_err!(context: 0u64 as bool);
+        should_err!(context: 1u64 as bool);
+        should_err!(context: 0.0f64 as bool);
+        should_err!(context: 1.0f64 as bool);
     }
 
     #[quickcheck]
